@@ -1,4 +1,4 @@
-/* $EPIC: newio.c,v 1.68 2008/04/04 04:51:05 jnelson Exp $ */
+/* $EPIC: newio.c,v 1.69 2011/04/02 07:59:42 crazyed Exp $ */
 /*
  * newio.c:  Passive, callback-driven IO handling for sockets-n-stuff.
  *
@@ -948,14 +948,24 @@ static	int	kdoit (Timeval *timeout)
 	    for (channel = 0; channel <= global_max_channel; channel++)
 	    {
 		/* 
-		 * We only ever do ONE event at a time, because new_io_event
-		 * could have any effect, including closing other fds!
+		 * We formerly only ever did ONE event at a time, because
+		 * new_io_event could have any effect, including closing other
+		 * fds!
+		 *
+		 * This is a performance issue when epic is under load though
+		 * and realistically will only ever occur if a script
+		 * disconnects a server from within a hook generated by another
+		 * server.
+		 *
+		 * In this case, putting the code in question in a /defer or
+		 * /timer is probably a better solution than uncommenting the
+		 * break.
 		 */
 		if (FD_ISSET(channel, &working_rd) ||
 		    FD_ISSET(channel, &working_wd))
 		{
 			new_io_event(VFD(channel));
-			break;
+			/* break; */
 		}
 	    }
 	}
