@@ -1,4 +1,4 @@
-/* $EPIC: notify.c,v 1.25 2003/11/20 23:38:13 jnelson Exp $ */
+/* $EPIC: notify.c,v 1.35 2008/03/10 06:29:39 crazyed Exp $ */
 /*
  * notify.c: a few handy routines to notify you when people enter and leave irc 
  *
@@ -36,8 +36,8 @@
 
 /*
  * Revamped by lynX - Dec '91
- * Expanded to allow notify on every server by hop, Feb 1996.
- * Mostly rewritten by jfn (esl), Dec 1997.
+ * Expanded to allow notify on every server in Feb 1996.
+ * Mostly rewritten in Dec 1997.
  */
 
 #define NEED_SERVER_LIST
@@ -165,7 +165,7 @@ BUILT_IN_COMMAND(notify)
 	int		do_ison = 0;
 	int		shown = 0;
 	NotifyItem	*new_n;
-	int		refnum;
+	int		refnum, first = 0, last = number_of_servers;
 	int		added = 0;
 	size_t		clue = 0;
 
@@ -178,12 +178,14 @@ BUILT_IN_COMMAND(notify)
 		if ((ptr = strchr(nick, ',')))
 		    *ptr++ = '\0';
 
-		if (*nick == '-')
+		if (0 <= from_server && !my_stricmp(":", nick))
+			first = last = from_server, last++;
+		else if (*nick == '-')
 		{
 		    nick++;
 		    if (*nick)
 		    {
-			for (refnum = 0; refnum < number_of_servers; refnum++)
+			for (refnum = first; refnum < last; refnum++)
 			{
 			    if (!(s = get_server(refnum)))
 				continue;
@@ -212,7 +214,7 @@ BUILT_IN_COMMAND(notify)
 		    }
 		    else
 		    {
-			for (refnum = 0; refnum < number_of_servers; refnum++)
+			for (refnum = first; refnum < last; refnum++)
 			{
 			    if (!(s = get_server(refnum)))
 				continue;
@@ -245,7 +247,7 @@ BUILT_IN_COMMAND(notify)
 			continue;
 		    }
 
-		    for (refnum = 0; refnum < number_of_servers; refnum++)
+		    for (refnum = first; refnum < last; refnum++)
 		    {
 			if (!(s = get_server(refnum)))
 			    continue;
@@ -277,13 +279,12 @@ BUILT_IN_COMMAND(notify)
 
 	if (do_ison && get_int_var(DO_NOTIFY_IMMEDIATELY_VAR))
 	{
-	    for (refnum = 0; refnum < number_of_servers; refnum++)
+	    for (refnum = first; refnum < last; refnum++)
 	    {
 		if (!(s = get_server(refnum)))
 		    continue;
 
-		if (is_server_registered(refnum))
-		    if (list && *list)
+		if (is_server_registered(refnum) && list && *list)
 			isonbase(refnum, list, ison_notify);
 	    }
 	}
@@ -342,10 +343,6 @@ void 	do_notify (void)
 	Server 		*s;
 	int 		old_from_server = from_server;
 	int		servnum;
-#if 0
-static	time_t		last_notify = 0;
-	int		interval = get_int_var(NOTIFY_INTERVAL_VAR);
-#endif
 
 	if (!number_of_servers)
 		return;
@@ -353,20 +350,6 @@ static	time_t		last_notify = 0;
 	if (x_debug & DEBUG_NOTIFY)
 		yell("do_notify() was called...");
 
-#if 0
-	if (time(NULL) < last_notify)
-		last_notify = time(NULL);
-	else if (!interval || interval > (time(NULL) - last_notify))
-	{
-		if (x_debug & DEBUG_NOTIFY)
-			yell("Not time for notify yet [%ld] [%ld]",
-				time(NULL), last_notify+interval);
-
-		return;		/* Not yet */
-	}
-	last_notify = time(NULL);
-#endif
-
 	for (servnum = 0; servnum < number_of_servers; servnum++)
 	{
 		if (!(s = get_server(servnum)))
@@ -381,11 +364,13 @@ static	time_t		last_notify = 0;
 		}
 
 		from_server = servnum;
-		if (NOTIFY_LIST(s)->ison && *NOTIFY_LIST(s)->ison)
+		if (NOTIFY_LIST(s)->ison && *NOTIFY_LIST(s)->ison
+				&& !get_server(servnum)->ison_wait)
 		{
 			isonbase(servnum, NOTIFY_LIST(s)->ison, ison_notify);
 			if (x_debug & DEBUG_NOTIFY)
-			    yell("Notify ISON issued for server [%d]", servnum);
+			    yell("Notify ISON issued for server [%d] with [%s]"
+				, servnum, NOTIFY_LIST(s)->ison);
 		}
 		else if (NOTIFY_MAX(s))
 		{
@@ -393,7 +378,7 @@ static	time_t		last_notify = 0;
 				yell("Server [%d]'s notify list is"
 					"empty and it shouldn't be.",
 						servnum);
-			rebuild_all_ison();
+			rebuild_notify_ison(servnum);
 		}
 		else if (x_debug & DEBUG_NOTIFY)
 			yell("Server [%d]'s notify list is empty.",
@@ -483,7 +468,7 @@ void 	dispatch_notify_userhosts (int refnum)
 	{
 		if (x_debug & DEBUG_NOTIFY)
 			yell("Dispatching notifies to server [%d], [%s]", from_server, batched_notify_userhosts);
-		userhostbase(refnum, batched_notify_userhosts, notify_userhost_dispatch, 1);
+		userhostbase(refnum, batched_notify_userhosts, NULL, notify_userhost_dispatch, 1);
 		new_free(&batched_notify_userhosts);
 		batched_notifies = 0;
 	}
@@ -512,7 +497,7 @@ void 	notify_userhost_reply (int refnum, const char *nick, const char *uh)
 	{
 		if (do_hook(NOTIFY_SIGNON_LIST, "%s %s", nick, uh))
 		{
-			if (!*uh)
+			if (*uh)
 				say("Signon by %s!%s detected", nick, uh);
 			else
 				say("Signon by %s detected", nick);
@@ -528,23 +513,6 @@ void 	notify_userhost_reply (int refnum, const char *nick, const char *uh)
 }
 
 
-void 	save_notify (FILE *fp)
-{
-	Server *s = NULL;
-	int i;
-
-	for (i = 0; i < number_of_servers; i++)
-		if ((s = get_server(i)) && NOTIFY_MAX(s))
-			break;
-	if (!s)
-		return;			/* No notify list. */
-
-	fprintf(fp, "NOTIFY");
-	for (i = 0; i < NOTIFY_MAX(s); i++)
-		fprintf(fp, " %s", NOTIFY_ITEM(s, i)->nick);
-	fprintf(fp, "\n");
-}
-
 void 	make_notify_list (int refnum)
 {
 	Server *s, *sp = NULL;
@@ -579,7 +547,7 @@ void 	make_notify_list (int refnum)
 		malloc_strcat_wordlist_c(&list, space, tmp->nick, &clue);
 	}
 
-	if (list)
+	if (list && !get_server(refnum)->ison_wait)
 	{
 		isonbase(refnum, list, ison_notify);
 		new_free(&list);
@@ -632,12 +600,12 @@ void	notify_systimer (void)
 	do_notify();
 }
 
-void    set_notify_interval (const void *stuff)
+void    set_notify_interval (void *stuff)
 {
 	update_system_timer(notify_timeref);
 }
 
-void    set_notify (const void *stuff)
+void    set_notify (void *stuff)
 {
 	update_system_timer(notify_timeref);
 }
