@@ -1,4 +1,4 @@
-/* $EPIC: term.c,v 1.15 2013/02/17 13:04:18 jnelson Exp $ */
+/* $EPIC: term.c,v 1.26 2008/11/26 03:26:34 jnelson Exp $ */
 /*
  * term.c -- termios and (termcap || terminfo) handlers
  *
@@ -47,6 +47,7 @@
 #include "window.h"
 #include "screen.h"
 #include "output.h"
+#include "newio.h"
 
 /*
  * If "HAVE_TERMINFO" is #define'd then we will use terminfo type function
@@ -65,8 +66,9 @@
 #include <sys/ioctl.h>
 
 volatile	int		need_redraw;
-static		int		tty_des;	/* descriptor for the tty */
-static		struct termios	oldb, newb;
+static	int		tty_des;		/* descriptor for the tty */
+static	struct	termios	oldb, newb;
+	char		my_PC;
 
 /*
  * XXX -- Supporting this correctly is a hack, but it's not my fault.
@@ -84,25 +86,27 @@ static		struct termios	oldb, newb;
  * using the system's (and not our) tparm().
  */
 #if defined(HAVE_TERM_H) && !defined(DONT_USE_TERM_H)
+/*
 # if defined(TERM_H_REQUIRES_CURSES_H)
 #  include <termio.h>
 #  include <curses.h>
 # endif
 # include <term.h>
+*/
 #else
 # ifdef HAVE_TERMINFO
-	extern	int	setupterm();
-	extern	char *	tigetstr();
-	extern	int	tigetnum();
-	extern	int	tigetflag();
+	extern  int     setupterm();
+	extern  char *  tigetstr();
+	extern  int     tigetnum();
+	extern  int     tigetflag();
 # else
-	extern	int	tgetent();
-	extern	char *	tgetstr();
-	extern	int	tgetnum();
-	extern	int	tgetflag();
+	extern  int     tgetent();
+	extern  char *  tgetstr();
+	extern  int     tgetnum();
+	extern  int     tgetflag();
 # endif
 # ifdef HAVE_TPARM
-	extern	char *	tparm();
+	extern  char *  tparm();
 # endif
 #endif
 
@@ -110,39 +114,38 @@ static		struct termios	oldb, newb;
  * We use macro wrappers to hide the difference between terminfo and termcap.
  */
 #ifdef HAVE_TERMINFO
-# define Tgetstr(x, y) 	tigetstr(x.iname)
-# define Tgetnum(x) 	tigetnum(x.iname);
-# define Tgetflag(x) 	tigetflag(x.iname);
+# define Tgetstr(x, y)	tigetstr(x.iname)
+# define Tgetnum(x)	tigetnum(x.iname);
+# define Tgetflag(x)	tigetflag(x.iname);
 #else
-# define Tgetstr(x, y) 	tgetstr(x.tname, &y)
-# define Tgetnum(x) 	tgetnum(x.tname)
-# define Tgetflag(x) 	tgetflag(x.tname)
+# define Tgetstr(x, y)	tgetstr(x.tname, &y)
+# define Tgetnum(x)	tgetnum(x.tname)
+# define Tgetflag(x)	tgetflag(x.tname)
 #endif
 
 /*
  * And for the final inanity, X/OPEN curses uses
- *	char *tparm (char *, int, int, int, int, int, int, int, int, int);
+ *     char *tparm (char *, int, int, int, int, int, int, int, int, int);
  * but everyone else (including ncurses and solaris) uses
- *	char *tparm (const char *, ...);
+ *     char *tparm (const char *, ...);
  * and there's no way to cover those two arglists with one prototype, so
  * we're left with these idiot wrapper functions.
  */
-char *tparm1 (const char *str, int l1)
+static char *tparm1 (const char *str, int l1)
 {
 	return tparm(str, l1, 0, 0, 0, 0, 0, 0, 0, 0);
 }
 
-char *tparm2 (const char *str, int l1, int l2)
+static char *tparm2 (const char *str, int l1, int l2)
 {
 	return tparm(str, l1, l2, 0, 0, 0, 0, 0, 0, 0);
 }
 
-char *tparm4 (const char *str, int l1, int l2, int l3, int l4)
+static char *tparm4 (const char *str, int l1, int l2, int l3, int l4)
 {
 	return tparm(str, l1, l2, l3, l4, 0, 0, 0, 0, 0);
 }
 
-
 /*
  * The old code assumed termcap. termcap is almost always present, but on
  * many systems that have both termcap and terminfo, termcap is deprecated
@@ -691,13 +694,14 @@ int	term_echo (int flag)
 }
 
 /*
- * term_putchar: This is what handles the outputting of the input buffer.
+ * term_inputline_putchar: This is what handles the outputting of the input 
+ * buffer.
  * It used to handle more, but slowly the stuff in screen.c prepared away
  * all of the nasties that this was intended to handle.  Well anyhow, all
  * we need to worry about here is making sure nothing suspcious, like an
  * escape, makes its way to the output stream.
  */
-void	term_putchar (unsigned char c)
+void	term_inputline_putchar (unsigned char c)
 {
 	if (!term_echo_flag)
 	{
@@ -705,19 +709,10 @@ void	term_putchar (unsigned char c)
 		return;
 	}
 
-	/*
-	 * If the user has /set eight_bit_characters off, then we lop off
-	 * any 8 bit characters we might stumble across.
-	 */
-	if (!(newb.c_cflag & CS8) && (c & 0x80))
-		c &= ~(0x80);
-
 	/*
 	 * Any nonprintable characters we lop off.  In addition to this,
 	 * we catch the very nasty 0x9b which is the escape character with
-	 * the high bit set.  It is possible (indeed, likely) that a terminal
-	 * with only 7 bit capabilities wont be reflected in /set eight_bit.
-	 * It is therefore our job to head that possibility off at the pass.
+	 * the high bit set.  
 	 */
 	if (c < 0x20 || c == 0x9b)
 	{
@@ -791,7 +786,7 @@ SIGNAL_HANDLER(term_cont)
 /*
  * term_pause: sets terminal back to pre-program days, then SIGSTOPs itself. 
  */
-void 	term_pause (char unused, char *not_used)
+BUILT_IN_KEYBINDING(term_pause)
 {
 	term_reset();
 	kill(getpid(), SIGSTOP);
@@ -815,7 +810,7 @@ int 	term_init (void)
 	char	*term;
 
 	if (dumb_mode)
-		panic("term_init called in dumb_mode");
+		panic(1, "term_init called in dumb_mode");
 
 	if ((term = getenv("TERM")) == (char *) 0)
 	{
@@ -884,24 +879,28 @@ int 	term_init (void)
 
 	current_term->TI_normal[0] = 0;
 	if (current_term->TI_sgr0)
-		strlcat(current_term->TI_normal, current_term->TI_sgr0, sizeof current_term->TI_normal);
+		strlcat(current_term->TI_normal, current_term->TI_sgr0, 
+				sizeof current_term->TI_normal);
 	if (current_term->TI_rmso)
 	{
 		if (current_term->TI_sgr0 && 
 		    strcmp(current_term->TI_rmso, current_term->TI_sgr0))
-			strlcat(current_term->TI_normal, current_term->TI_rmso, sizeof current_term->TI_normal);
+			strlcat(current_term->TI_normal, current_term->TI_rmso,
+					 sizeof current_term->TI_normal);
 	}
 	if (current_term->TI_rmul)
 	{
 		if (current_term->TI_sgr0 && 
 		    strcmp(current_term->TI_rmul, current_term->TI_sgr0))
-			strlcat(current_term->TI_normal, current_term->TI_rmul, sizeof current_term->TI_normal);
+			strlcat(current_term->TI_normal, current_term->TI_rmul, 
+					sizeof current_term->TI_normal);
 	}
 	/*
 	 * On some systems, alternate char set mode isn't exited with sgr0
 	 */
 	if (current_term->TI_rmacs)
-		strlcat(current_term->TI_normal, current_term->TI_rmacs, sizeof current_term->TI_normal);
+		strlcat(current_term->TI_normal, current_term->TI_rmacs, 
+				sizeof current_term->TI_normal);
 
 
 	/*
@@ -1002,7 +1001,7 @@ int 	term_init (void)
 
 	if ((termfeatures & desired) != desired)
 	{
-		fprintf(stderr, "\nYour terminal (%s) cannot run IRC II in full screen mode.\n", term);
+		fprintf(stderr, "\nYour terminal (%s) cannot run EPIC in full screen mode.\n", term);
 		fprintf(stderr, "The following features are missing from your TERM setting.\n");
 		if (!(termfeatures & TERM_CAN_CUP))
 			fprintf (stderr, "\tCursor movement\n");
@@ -1064,18 +1063,56 @@ int 	term_init (void)
 	tty_des = 0;			/* Has to be. */
 	tcgetattr(tty_des, &oldb);
 
+	/*
+	 * So by default, the kernel intercepts a great deal of keypresses
+	 * and does in-kernel input editing.  These keypresses are handled
+	 * by the kernel, and the result is a line of input which is then 
+	 * passed to the application.  Those special keypresses are never
+	 * sent to the application.
+	 *
+	 * So we need to turn all that off, since we need to be able to 
+	 * bind anything.
+	 */
+
+	/*
+	 * Turning off ICANON tells the kernel not to buffer input from
+	 * the user, but to pass it on to us directly.  Further, it tells
+	 * the kernel not to process the EOF (^D), EOL (^J), ERASE (^?)
+	 * and KILL (^U) keys, but to pass them on.
+	 *
+	 * Turning off ECHO tells the kernel not to output characters as
+	 * they are typed; we take responsibility for that.
+	 */
 	newb = oldb;
 	newb.c_lflag &= ~(ICANON | ECHO); /* set equ. of CBREAK, no ECHO */
-#ifdef IEXTEN
-	if (use_iexten == 1)
-		newb.c_lflag |= IEXTEN;	    /* Turn off DISCARD/LNEXT */
-	else if (use_iexten == 0)
-		newb.c_lflag &= ~(IEXTEN);  /* Turn off DISCARD/LNEXT */
-#endif
 
+	/*
+	 * Turning off IEXTEN tells the kernel not to honor any posix 
+	 * extenstions it might know about.  On 4.4BSD, this tells the 
+	 * kernel not to withhold the EOL2 (undefined), WERASE (^W), 
+	 * REPRINT (^R), LNEXT (^V), DISCARD (^O), and STATUS (^T) 
+	 * characters from us.  You wouldn't be able to /bind those keys
+	 * if this was left on.
+	 */
+	newb.c_lflag &= ~(IEXTEN);  /* Turn off DISCARD/LNEXT */
+
+	/*
+	 * Since we turned off ICANON, we have to tell the kernel how
+	 * we want the read() call to be satisifed.  We want read() to
+	 * return 0 seconds after 1 byte has been received (ie, immediately)
+	 */
 	newb.c_cc[VMIN] = 1;	          /* read() satified after 1 char */
 	newb.c_cc[VTIME] = 0;	          /* No timer */
 
+	/*
+	 * Next, we have to reclaim all of the signal-generating keys.
+	 * Normally you would do that by turning off ISIG, but we can't do
+	 * that, because we want ^C to generate SIGINT; that's how the user
+	 * can bail out of infinite recursion.
+	 *
+	 * So we have to actually turn off all of the signal-generating
+	 * keys *except* ^C by hand:  QUIT (^\), DSUSP (^Y), and SUSP (^Z).
+	 */
 #       if !defined(_POSIX_VDISABLE)
 #               if defined(HAVE_FPATHCONF)
 #                       define _POSIX_VDISABLE fpathconf(tty_des, _PC_VDISABLE)
@@ -1084,10 +1121,6 @@ int 	term_init (void)
 #               endif
 #       endif
 
-	/* 
-	 * We can't just turn off ISIG since we want ^C to send SIGINT,
-	 * so we turn off all of the other signal-generating keys
-	 */
 	newb.c_cc[VQUIT] = _POSIX_VDISABLE;
 #	ifdef VDSUSP
 		newb.c_cc[VDSUSP] = _POSIX_VDISABLE;
@@ -1096,19 +1129,55 @@ int 	term_init (void)
 		newb.c_cc[VSUSP] = _POSIX_VDISABLE;
 #	endif
 
-	if (!use_flow_control)
-		newb.c_iflag &= ~(IXON);	/* No XON/XOFF */
+	/*
+	 * Now we have to turn off hardware flow control, in order to reclaim
+	 * the ^S (XOFF) and ^Q (XON) keys.  Hardware flow control is evil,
+	 * because while the flow control is off, epic will block when it
+	 * tries to output something, which could lead to you pinging out 
+	 * all your servers.
+	 */
+	newb.c_iflag &= ~(IXON | IXOFF);	/* No XON/XOFF */
 
 #if use_alt_screen
-	/* Ugh. =) This is annoying! */
+	/*
+	 * Some terminal emulators have two screens; the primary screen,
+	 * and the alternate screen.  Applications can optionally switch
+	 * to the alternate screen at startup, and then switch back to the
+	 * primary screen at shutdown.  I find this feature to be intensely
+	 * obnoxious.  It's here, if you want to #define it.
+	 */
 	if (current_term->TI_smcup)
 		tputs_x(current_term->TI_smcup);
 #endif
+
 #if use_automargins
+	/*
+	 * When you write a character to the last column of a line, most
+	 * terminal emulators will advance the cursor to the start of the
+	 * next line ("automargins") but some will leave the cursor at the end
+	 * of the line ("no automargins").  It's not always obvious which
+	 * state you're in, and it's not always obvious whether the emulator
+	 * supports one, or both modes.
+	 *
+	 * The client avoids all this by refusing to use the last column.  
+	 * But if you want to, you can define this and epic will turn off 
+	 * auto-margins and will attempt to use the final column in each line.
+	 * There are no guarantees it will work, since some emulators say they 
+	 * support turning off auto-margins, but they lie.
+	 */
 	if (current_term->TI_rmam)
 		tputs_x(current_term->TI_rmam);
 #endif
 
+	/*
+	 * Next we tell the kernel that it should support 8 bits both
+	 * coming in and going out, and it should not strip the high bit
+	 * from any keypress.
+	 */
+	newb.c_cflag |= CS8;
+	newb.c_iflag &= ~ISTRIP;
+
+	/* Commit our changes and we're done! */
 	tcsetattr(tty_des, TCSADRAIN, &newb);
 	return 0;
 }
@@ -1129,6 +1198,22 @@ int	term_resize (void)
 	{
 		struct winsize window;
 
+		/*
+		 * This hack is required by a race condition within screen;
+		 * if you have a "caption always" bar, when you reattach to
+		 * a session, it will send us a SIGWINCH, before it has 
+		 * accounted for the "caption bar" stealing a line from the
+		 * screen.  So we race screen, and if we ask for the size of
+		 * the screen before it accounts for the caption bar, then
+		 * we lose, because we'll get the wrong number of lines, and
+		 * that will screw up the status bar.  So we do this very
+		 * small sleep to increase the chances of us losing the race,
+		 * which means we win.  Got it?
+		 *
+		 * P.S. -- GNU Screen is all kinds of icky.
+		 */
+		my_sleep(0.05);
+
 		if (ioctl(tty_des, TIOCGWINSZ, &window) < 0)
 		{
 			current_term->TI_lines = li;
@@ -1187,41 +1272,13 @@ void	term_beep (void)
 	}
 }
 
-static int	orig_term_eight_bit (void)
+void	set_meta_8bit (void *stuff)
 {
-	if (dumb_mode)
-		return 1;
-	return (((oldb.c_cflag) & CSIZE) == CS8) ? 1 : 0;
-}
+	VARIABLE *v;
+	int	value;
 
-int 	term_eight_bit (void)
-{
-	if (dumb_mode)
-		return 1;
-	return (((newb.c_cflag) & CSIZE) == CS8) ? 1 : 0;
-}
-
-void	set_term_eight_bit (int value)
-{
-	if (dumb_mode)
-		return;
-
-	if (value == ON)
-	{
-		newb.c_cflag |= CS8;
-		newb.c_iflag &= ~(ISTRIP);
-	}
-	else
-	{
-		newb.c_cflag &= ~(CS8);
-		newb.c_iflag |= ISTRIP;
-	}
-	tcsetattr(tty_des, TCSADRAIN, &newb);
-}
-
-void	set_meta_8bit (const void *stuff)
-{
-	int	value = *(const int *)stuff;
+	v = (VARIABLE *)stuff;
+	value = v->integer;
 
 	if (dumb_mode)
 		return;
@@ -1361,7 +1418,7 @@ void	term_insert (unsigned char c)
 	else if (current_term->TI_ich)
 		tputs_x (tparm1(current_term->TI_ich, 1));
 
-	term_putchar (c);
+	term_inputline_putchar (c);
 
 	if (current_term->TI_rmir)
 		tputs_x(current_term->TI_rmir);
@@ -1604,7 +1661,7 @@ const char *	term_getsgr (int opt, int fore, int back)
 				ret = tparm1(current_term->TI_dispc, fore);
 			break;
 		default:
-			panic ("Unknown option '%d' to term_getsgr", opt);
+			panic(1, "Unknown option '%d' to term_getsgr", opt);
 			break;
 	}
 	return (ret);
@@ -1612,7 +1669,7 @@ const char *	term_getsgr (int opt, int fore, int back)
 
 static char *	control_mangle (unsigned char *text)
 {
-static 	u_char	retval[256];
+static 	unsigned char	retval[256];
 	int 	pos = 0;
 	
 	*retval = 0;
@@ -1679,3 +1736,123 @@ static	char		retval[128];
 	return NULL;	
 }
 
+#ifdef WITH_THREADED_STDOUT
+
+#include <pthread.h>
+#include "tio.h"
+
+#define TIOBUFSZ	2048
+
+typedef struct tio_qentry_stru {
+	tio_file	*file;
+	int		 len, flush, close;
+struct	tio_qentry_stru	*next, *tail;
+	char 		 buf[TIOBUFSZ];
+} tio_qentry;
+
+struct tio_file_stru {
+	FILE		*stdio_file;
+	tio_qentry	*buf;
+};
+
+static void		tio_push(tio_file *);
+static tio_qentry	*new_tiobuf(tio_file *);
+
+tio_file	*tio_stdout;
+
+static	tio_qentry	tio_qhead;
+static	pthread_cond_t	tio_cond = PTHREAD_COND_INITIALIZER;
+static	pthread_mutex_t	tio_mtx = PTHREAD_MUTEX_INITIALIZER;
+static	pthread_t	tio_thread;
+
+static tio_qentry	*new_tiobuf(tio_file *f)
+{
+	tio_qentry	*ret;
+	ret = calloc(1, sizeof(*ret));
+	ret->file = f;
+	return ret;
+}
+
+void	tio_fputc(int c, tio_file *f)
+{
+	if (f->buf->len == TIOBUFSZ)
+		tio_push(f);
+	f->buf->buf[f->buf->len++] = c;
+}
+
+void	tio_fputs(const char *str, tio_file *stream)
+{
+	while (*str)
+		tio_fputc(*str++, stream);
+}
+
+tio_file	*tio_open(FILE *f)
+{
+	tio_file	*ret;
+
+	if (f == NULL)
+		return NULL;
+
+	ret = calloc(1, sizeof(*ret));
+	ret->stdio_file = f;
+	ret->buf = new_tiobuf(ret);
+	return ret;
+}
+
+void	tio_flush(tio_file *stream)
+{
+	stream->buf->flush = 1;
+	tio_push(stream);
+}
+
+static void	tio_push(tio_file *stream)
+{
+	pthread_mutex_lock(&tio_mtx);
+	if (!tio_qhead.next) {
+		tio_qhead.next = tio_qhead.tail = stream->buf;
+	} else {
+		tio_qhead.tail->next = stream->buf;
+		tio_qhead.tail = stream->buf;
+	}
+
+	pthread_cond_signal(&tio_cond);
+	pthread_mutex_unlock(&tio_mtx);
+
+	stream->buf = new_tiobuf(stream);
+}
+
+static void	*tio_thread_run(void *unused)
+{
+	for (;;) {
+		tio_qentry	*tq, *next;
+
+		pthread_mutex_lock(&tio_mtx);
+		if (tio_qhead.next == NULL)
+			pthread_cond_wait(&tio_cond, &tio_mtx);
+		tq = tio_qhead.next;
+		tio_qhead.next = tio_qhead.tail = NULL;
+		pthread_mutex_unlock(&tio_mtx);
+
+		for (next = tq ? tq->next : NULL; tq;
+		     tq = next, next = tq ? tq->next : NULL) {
+			if (tq->len)
+				fwrite(tq->buf, 1, tq->len, tq->file->stdio_file);
+			if (tq->flush)
+				fflush(tq->file->stdio_file);
+			free(tq);
+		}
+	}
+}
+
+void	tio_init(void)
+{
+	tio_stdout = tio_open(stdout);
+	pthread_create(&tio_thread, NULL, tio_thread_run, NULL);
+}
+
+void	tio_close (tio_file *stream)
+{
+	yell("Need to implement tio_close!");
+}
+
+#endif
