@@ -1,4 +1,4 @@
-/* $EPIC: vars.c,v 1.108 2010/04/29 01:41:23 jnelson Exp $ */
+/* $EPIC: vars.c,v 1.109 2010/05/30 01:12:01 jnelson Exp $ */
 /*
  * vars.c: All the dealing of the irc variables are handled here. 
  *
@@ -750,7 +750,7 @@ BUILT_IN_COMMAND(setcmd)
 
 		if (b->numitems == 0)
 		{
-		    if (!do_hook(UNKNOWN_SET_LIST, "%s %s", 
+		    if (do_hook(UNKNOWN_SET_LIST, "%s %s", 
 					var, args ? args : "<unset>"))
 		      if (do_hook(SET_LIST, "set-error No such variable \"%s\"",
 					var))
